/*
 * MudStorm OS
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <types.h>
#include <console.h>
#include <video.h>
#include <shell.h>
#include <stdio.h>
#include <rtc.h>
#include <delay.h>
#include <version.h>
#include <string.h>
#include <speaker.h>

int DEBUG = 0; // Inzializzo una variabile "int" con nome "DEBUG" con valore "0"
string command = NULL; // Inizializzo una variabile "command" come "string", con valore "NULL"

void help()
{
	if (DEBUG) printk("\nScrivo i comandi possibili a video\n"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	for (int i=0;i<NUM;i++) // Conto fino al valore della variabile NUM(definita nel file shell.h)
	{
	printk("\n %s - %s", shellcmd[i].cmdname, shellcmd[i].cmddesc); // Scrivo a schermo: nome del comando - descrizione
	}
	printk("\n"); // Vado a capo
}

void clear()
{
	if (DEBUG) printk("\nPulisco lo schermo"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	cls(); // Pulisco lo schermo
	if (DEBUG) printk("\nStampo a video il logo\n"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	logo(); // Stampo a video il logo
}

void uname()
{
	if (DEBUG) printk("\nStampo a video le informazioni dell'OS\n"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	set_color(BLUE); // Le prossime scritte saranno blu
	printk("\n%s%s, %s: %s - %s: %s\n%s: %s\n", "MudStormOS", REVISION, "Kernel", KERNEL, "Shell", SHELL, "Autore", AUTHOR);
	// Scrivo a schermo "MudStormOS-Revisione, Kernel:Versione - Shell:Versione a capo Autore: Autore"
	set_color(WHITE); // Le prossime scritte saranno bianche
}

void date()
{
	if (DEBUG) printk("\nLeggo le informazioni dall'RTC"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	read_rtc(); // Leggo dall'RTC
	if (DEBUG) printk("\nScrivo le informazioni ricevute dall'RTC\n"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	printk("\n %d:%d:%d - %d/%d/%d\n", hour, minute, second, day, month, year); 
	// Scrivo a schermo "ora:minuti:secondi - giorno/mese/anno"
}

void cpu()
{
	if (DEBUG) printk("\nScrivo a video la velocità (attuale) della CPU\n"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	int speed = cpu_speed(); // Inzializzo una variabile "speed" come "int" con valore(preso da una funzione) "cpu_speed()"
	printk("\n%d (MHz)\n", speed); // Scrivo a video
}

void beepp()
{
	if (DEBUG) printk("\nFaccio suonare il buzzer\n"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	beep(); // Richiamo la funzione beep();
}

void debug()
{
	if (DEBUG) // Se la variabile "DEBUG" ha valore "1"
	{
		DEBUG = 0; // "DEBUG" cambia valore
		printk("\nDebug disabilitato\n"); // Scrivo a video
	}
	else // // Se la variabile "DEBUG" ha valore "0"
	{
		DEBUG = 1; // "DEBUG" cambia valore
		printk("\nDebug abilitato\n"); // Scrivo a video
	}
}

void logos()
{
	if (DEBUG) printk("\nStampo a video il logo"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	logo(); // Richiamo la funzione logo();
}

void reb()
{
	if (DEBUG) printk("\nRiavvio la macchina"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	reboot(); // Richiamo la funzione reboot();
}

void shut()
{
	if (DEBUG) printk("\nArresto la macchina"); // Se la variabile "DEBUG" ha valore "1", allora scrivi a schermo
	shutdown(); // Richiamo la funzione shutdown();
}

struct cmd shellcmd[NUM] = { // Dichiaro una struttura con tutti questi comandi
{ "help", help, "Mostra i comandi" },
{ "clear", clear, "Pulisce lo schermo" },
{ "uname", uname, "Mostra le informazioni sull'OS" },
{ "date", date, "Mostra la data" },
{ "cpu", cpu, "Mostra la velocita' attuale della CPU" },
{ "beep", beepp, "Suona col buzzer" },
{ "debug", debug, "Controlla cosa fa la shell" },
{ "logo", logos, "Stampa a schermo il logo" },
{ "reboot", reb, "Riavvia la macchina" },
{ "shutdown", shut, "Arresta la macchina" },
};

int find(string cmd)
{
	for(int i=0;i<NUM;i++) // Conto fino al valore della variabile NUM(definita nel file shell.h
	{
		if(!strcmp(shellcmd[i].cmdname, cmd)) 
			return i; // Se il strcmp trova una stringa uguale sia nella stringa cmd, sia nella string shellcmd[i].cmdname, allora ritorna il numero in cui l'ha trovata
	}
	return -1; // Altrimenti ritorna -1 (not found)
}

void runShell()
{
	//cls(); // Pulisco lo schermo
	logo(); // Stampo a video il logo
	set_color(BAK_WHITE); // Le prossime scritte saranno nere con sfondo bianco
	printk("\t\t\t\t<help>\n"); // Scrivo a video
	set_color(WHITE); // Le prossime scritte saranno bianche
	
	while(1) // Ciclo infinito
	{
		printk("\n> "); // Scrivo a video
		scank("%s", command); // Leggo quello che scrivi come una stringa

		int i = find(tolower(command)); // Inizializzo una variable "i" come "int" con valore "find(command)"
		if (i >= 0) // Controllo se la variabile "i" è maggiore o uguale a "0"
		{
			void (*commandJump)(void); // Inizializzo una variabile "*commandJump" come void
			commandJump = shellcmd[i].v_func; // Gli do valore shellcmd[i].v_func
			commandJump(); // Richiamo la funzione contenuta in shellcmd[i].v_func
		}
		else
		{
			printk("\nComando non trovato\n");
		}
	}
}
