#!/bin/sh

## NASM Compile
# nasm -f elf32 loader/<name_file_input.asm> -o $build/<name_file_output.o>

## C Compile
# gcc $opt <name_file_input.c> -o $build/<name_file_output.o>

## LD Link
# ld -m elf_i386 -T <name_file_link.ld> -o <name_file_output.bin> $build/<name_files_input.o>

echo "Setto i tools..."
export PATH=${PATH}:/usr/local/bin
build=objs
img=out
temp=tmp
opt="-m32 -Wall -O -fstrength-reduce -fomit-frame-pointer -nostdinc -fno-builtin -I./kernel/include -fno-strict-aliasing -fno-common -std=gnu99 -fno-stack-protector -c"

echo "Creo le cartelle..."
mkdir -p $build || exit
mkdir -p $temp || exit

echo "Compilazione (ASM) Loader..."
# ASM Loader
nasm -f elf32 loader/loader.S -o $build/loader.o

echo "Compilazione (C) Kernel..."
# C dmain
gcc $opt kernel/kernel.c -o $build/kernel.o

echo "Compilazione (C) Shell..."
# C shell
gcc $opt kernel/shell/shell.c -o $build/shell.o
echo "Compilazione (C) Misc..."
# C misc
gcc $opt kernel/misc/video.c -o $build/video.o
gcc $opt kernel/misc/console.c -o $build/console.o
gcc $opt kernel/misc/io.c -o $build/io.o
echo "Compilazione (C) Lib..."
# C lib
gcc $opt kernel/lib/random.c -o $build/random.o
gcc $opt kernel/lib/string.c -o $build/string.o
gcc $opt kernel/lib/timer.c -o $build/timer.o
gcc $opt kernel/lib/rtc.c -o $build/rtc.o 
gcc $opt kernel/lib/speaker.c -o $build/speaker.o
gcc $opt kernel/lib/stdio.c -o $build/stdio.o
gcc $opt kernel/lib/rdtsc.c -o $build/rdtsc.o

echo "Compilazione (C) k(ey)b(oar)d..."
# C k(ey)b(oar)d
gcc $opt kernel/kbd/kb.c -o $build/kb.o

echo "Compilazione (C) int(e)r(rupt)..."
# C int(e)r(rupt)
gcc $opt kernel/intr/gdt.c -o $build/gdt.o
gcc $opt kernel/intr/idt.c -o $build/idt.o
gcc $opt kernel/intr/irq.c -o $build/irq.o
gcc $opt kernel/intr/isrs.c -o $build/isrs.o

echo "Compilazione terminata"

echo "Linking..."
# LD Linking
ld -m elf_i386 -T linker/linker.ld -o kernel.bin $build/loader.o $build/kernel.o $build/shell.o $build/video.o $build/console.o $build/io.o $build/string.o $build/random.o $build/gdt.o $build/idt.o $build/irq.o $build/isrs.o $build/kb.o $build/timer.o $build/speaker.o $build/stdio.o $build/rtc.o $build/rdtsc.o

echo "Linking terminato"

echo "Richiedo permessi per smontare(per sicurezza)"
sudo umount $temp

echo "Richiedo permessi per montare"
sudo mount -o loop $img/grub+mudstorm.img $temp || exit

sleep 0.5 || exit

echo "Richiedo permessi per copiare il file"
sudo cp kernel.bin $temp/kernel.bin || exit

sleep 0.2 || exit

echo "Richiedo i permessi per smontare"
sudo umount $temp || exit

sleep 0.5 || exit

echo "Elimino il file *.bin"
rm kernel.bin || exit

echo "Elimino files *.o"
rm $build/*.o || exit

echo "Elimino le cartelle create"
rmdir $build || exit
rmdir $temp || exit


echo "Completato"

echo "Avvio OS con QEMU"
qemu-system-i386 -fda $img/grub+mudstorm.img
